/*		MANDELBROT INSPECTION PROGRAM
					BY
			   CHRIS SPARKMAN
*/

/////////////////////////////////////////////////////////////
///////////// INCLUDES //////////////////////////////////////
/////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <time.h>
#include <iostream.h>
#include "allegro.h"
#include "myfonts.h"

/////////////////////////////////////////////////////////////
///////////// FUNCTION PROTOTYPES ///////////////////////////
/////////////////////////////////////////////////////////////

void plot(double, double, BITMAP *bmp);
void plotJulia(double, double);
void drawAxes(void);
void drawMandelbrot(void);
void drawOrbit(double, double, BITMAP *bmp);
void drawJulia(double, double, BITMAP *bmp);
void getScreenResolution(void);
void getCursor(void);
double trueX(int);
double trueY(int);
char *listbox_getter(int index, int *list_size);

/////////////////////////////////////////////////////////////
///////////// GLOBAL VARIABLES //////////////////////////////
/////////////////////////////////////////////////////////////
#define LISTBOX_OBJECT    2

int xMax = 1024;
int yMax = 768;
double xLeft = -2.0,
	   xRight = .6,
	   yTop = (xRight - xLeft) * .75 / 2,
	   yBottom = -1.0 * yTop,
	   xRange = xRight - xLeft,
	   yRange = yTop - yBottom,
	   xLeftJulia = -2.0,
	   xRightJulia = 2.3,
	   yTopJulia = (xRightJulia - xLeftJulia) * .75/2 ,
	   yBottomJulia = -1.0 * yTopJulia,
	   xRangeJulia = xRightJulia - xLeftJulia,
	   yRangeJulia = yTopJulia - yBottomJulia,
	   currJuliaX = 0,
	   currJuliaY = 0;

int colorNum;
char msg[80];
char the_string[32] = "Choose the screen resolution.";
int xCursor;
int yCursor;
int fontSize;
int numFont;
BITMAP *orbit;
BITMAP *Julia;
BITMAP *bigJulia;
BITMAP *Mandelbrot;
BITMAP *coords;
BITMAP *custom_cursor;

DIALOG the_dialog[] =
{
   /* (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key) (flags)  (d1)                    (d2)  (dp)              (dp2) (dp3) */
   { d_clear_proc,      0,    0,    0,    0,    43 ,  22,   0,    0,       0,                      0,    NULL,             NULL, NULL  },
   { d_ctext_proc,      320,  150,  512,  48,   43 ,  22,   0,    0,       sizeof(the_string)-1,   0,    the_string,       NULL, NULL  },
   { d_list_proc,       260,  190,  120,  100,  43 ,  22,   0,    0,       0,                      0,    listbox_getter,   NULL, NULL  },
   { d_button_proc,     280,  300,  80,   30,   43 ,  22,   0,    D_EXIT,  0,                      0,    "OK",             NULL, NULL  },
   { NULL,              0,    0,    0,    0,    0,    0,    0,    0,       0,                      0,    NULL,             NULL, NULL  }
};

/////////////////////////////////////////////////////////////
/////////////BEGINNING OF MAIN///////////////////////////////
/////////////////////////////////////////////////////////////

int main(int argc, char *argv[])
{


	DATAFILE *datafile;   //datafile will contain my fonts
	char buf[256];

    srand(time(NULL));	  //seed the random number generator(for the colors)

    allegro_init();
    install_keyboard();
    install_mouse();
    install_timer();

/////////////select a screen resolution///////////////////////////////
    set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0);
    getScreenResolution();

/////////////set the screen resolution///////////////////////////////
    set_gfx_mode(GFX_AUTODETECT, xMax, yMax, 0, 0);

/////////////get the fonts//////////////////////////////////////////
    replace_filename(buf, argv[0], "myfonts.dat", sizeof(buf));
    datafile = load_datafile(buf);

/////////////set up the bitmaps and build Mandelbrot Set///////////////////////////////
	clear_to_color(screen, 255);
	sprintf(msg, "Building Mandelbrot Set . . . please wait.");
	textout_centre(screen, (FONT*)datafile[fontSize].dat, msg, xMax/2, yMax/2, 43);
	sprintf(msg, "At any time, you may press ESCAPE to quit.");
	textout_centre(screen, (FONT*)datafile[fontSize].dat, msg, xMax/2, yMax*6/10, 43);

	orbit = create_bitmap(xMax/3,yMax/3);
	Julia = create_bitmap(xMax/3,yMax/3);
	Mandelbrot = create_bitmap(xMax, yMax);
	bigJulia = create_bitmap(xMax, yMax);
	coords = create_bitmap(orbit->w * 2 / 3, orbit->h/3);

	clear_to_color(Mandelbrot, 255);
	clear_to_color(orbit, 255);
	clear_to_color(Julia, 255);
	clear_to_color(bigJulia, 255);
	clear_to_color(coords, 255);

	colorNum = 15;

	drawMandelbrot();

/////////////beginning of mouse features///////////////////////////////
	sprintf(msg, "Left-click for Julia Set");
	textout(Mandelbrot, (FONT*)datafile[fontSize - 2].dat, msg, 0, yMax * 95/100, 43);
	sprintf(msg, "Move the mouse");
	textout(Mandelbrot, (FONT*)datafile[fontSize - 1].dat, msg, 0, 0, 43);
	blit(Mandelbrot, screen, 0, 0, 0, 0, Mandelbrot->w, Mandelbrot->h);

	sprintf(msg, "Left-click for Julia Set");
	textout(screen, (FONT*)datafile[fontSize - 2].dat, msg, 0, yMax * 95/100, 43);

	getCursor();
	xCursor = mouse_x;
	yCursor = mouse_y;

	text_mode(-1);

	do
	{

		if (xCursor != mouse_x || yCursor != mouse_y)
		{
			show_mouse(NULL);
			/////////////draw orbit based on current mouse location///////////////////////////////
      		drawOrbit(trueX(mouse_x), trueY(mouse_y), orbit);

      		/////////////show coordinates of mouse cursor///////////////////////////////
      		clear_to_color(coords, 255);
			sprintf(msg, "x = %-10f", trueX(mouse_x));
			textout(coords, (FONT*)datafile[numFont].dat, msg, 0 , 0, 15);
			sprintf(msg, "y = %-10f", trueY(mouse_y));
			textout(coords, (FONT*)datafile[numFont].dat, msg, 0, coords->h/2, 15);

			blit(coords, screen, 0,0,0, orbit->h, coords->w, coords->h);
      		blit(orbit,screen, 0,0,0,0,orbit->w, orbit->h);

      		show_mouse(screen);

      		xCursor = mouse_x;
      		yCursor = mouse_y;
		}

		if (mouse_b & 1)
		{
			/////////////draw Julia set based on mouse position///////////////////////////////
			show_mouse(NULL);
			currJuliaX = trueX(mouse_x);
			currJuliaY = trueY(mouse_y);
      		drawJulia(trueX(mouse_x), trueY(mouse_y), Julia);
      		sprintf(msg, "Right-click mouse to magnify");
			textout_centre(Julia, (FONT*)datafile[fontSize-2].dat, msg, Julia->w/2, Julia->h*9/10, 43);
      		blit(Julia,screen, 0,0,0, yMax - Julia->h, Julia->w, Julia->h);
      		show_mouse(screen);
		}

		if (mouse_b & 2)
		{
			/////////////zoom in on Julia set///////////////////////////////
			show_mouse(NULL);
      		drawJulia(currJuliaX, currJuliaY, bigJulia);
      		blit(bigJulia,screen, 0,0,0,0, bigJulia->w, bigJulia->h);
      		sprintf(msg, "Press the space bar to continue");
			textout_centre(screen, (FONT*)datafile[fontSize].dat, msg, xMax/2, yMax*9/10, 43);
      		readkey();
      		blit(Mandelbrot, screen, 0, 0, 0, 0, Mandelbrot->w, Mandelbrot->h);

      		show_mouse(screen);
		}


    }while(!key[KEY_ESC]);

    destroy_bitmap(Mandelbrot);
  	destroy_bitmap(orbit);
  	destroy_bitmap(Julia);
  	destroy_bitmap(bigJulia);
  	unload_datafile(datafile);

    allegro_exit();
    return 0;
}

/////////////////////////////////////////////////////////////
///////////////////END OF MAIN///////////////////////////////
/////////////////////////////////////////////////////////////

void plot(double x, double y, BITMAP *bmp)
{
	int x1, y1;

	x1 = (xLeft - x)*(-1.0) * (double)(bmp->w)/xRange;
    y1 = (yTop - y) * (double)(bmp->h)/yRange;

    putpixel(bmp, x1,y1, colorNum);
}

void plotJulia(double x, double y, BITMAP *bmp)
{
	int x1, y1;

	x1 = (xLeftJulia - x)*(-1.0) * (double)(bmp->w)/xRangeJulia;
    y1 = (yTopJulia - y) * (double)(bmp->h)/yRangeJulia;

    putpixel(bmp, x1,y1 , colorNum);
}


void drawAxes(void)
{
	int x1, x2, y1, y2;
    x1 = xLeft*(-1) * xMax/xRange;
	y1 = 0;
	y2 = yMax;
    line(screen, x1,y1,x1,y2,35);

    x1 = 0;
    x2 = xMax;
    y1 = (yRange + yBottom)*yMax/yRange;
    line(screen, x1,y1,x2,y1,35);
}

void drawMandelbrot(void)
{
	double x, y, c, d;
	for (double i = xLeft; i <= xRight; i+= xRange/xMax)
	for (double j = yBottom; j <= yTop; j+= yRange/yMax)
	{
		x = i;
		y = j;
		for(int k = 1; k <= 150; k++)
		{
			c = x*x - y*y + i;
			d = 2*x*y + j;
			if (c > 5 || d > 5)
				break;
			x = c;
			y = d;
		}
		if (!(c > 5 || d > 5))
		{
			colorNum = 15;
			plot(i, j, Mandelbrot);
		}
	}
}

void drawJulia(double a, double b, BITMAP *bmp)
{
	double c, d;
	clear_to_color(bmp, 255);
	for (double i = xLeftJulia; i <= xRightJulia; i+= xRangeJulia/bmp->w)
	for (double j = yBottomJulia; j <= yTopJulia; j+= yRangeJulia/bmp->h)
	{
		double x, y;
		x = i;
		y = j;
		for(int k = 1; k <= 150; k++)
		{
			c = x*x - y*y + a;
			d = 2*x*y + b;
			if (c > 5 || d > 5)
				break;
			x = c;
			y = d;
		}
		if (!(c > 5 || d > 5))
			plotJulia(i , j, bmp);
	}
}

double trueX(int x)
{
	return mouse_x * xRange / xMax + xLeft;
}

double trueY(int y)
{
	return yTop - mouse_y * yRange / yMax;
}

void drawOrbit(double x, double y, BITMAP *bmp)
{
	double a, b, c, d;
	clear_to_color(bmp, 255);
	a = x;
	b = y;
	for (int i = 1; i <= 10000; i++)
	{
		c = a*a - b*b + x;
		d = 2*a*b + y;

		if (c > 5 || d > 5)
			break;
		if (i % 500 == 0)
			colorNum = rand() % 15 + 1;

		plot(c - .5, d, bmp);

		a = c;
		b = d;
	}
}

void getScreenResolution(void)
{
	int choice, ret;

	ret = do_dialog(the_dialog, -1);
	choice =  the_dialog[LISTBOX_OBJECT].d1;
	switch (choice)
	{
		case 0:
			xMax = 640;
			yMax = 480;
			fontSize = TEXT_20;
			numFont = NUM_12;
			break;
		case 1:
			xMax = 800;
			yMax = 600;
			fontSize = TEXT_24;
			numFont = NUM_16;
			break;
		case 2:
			xMax = 1024;
			yMax = 768;
			fontSize = TEXT_28;
			numFont = NUM_20;
			break;
		case 3:
			xMax = 1280;
			yMax = 1024;
			fontSize = TEXT_36;
			numFont = NUM_24;
			break;
		case 4:
			xMax = 1600;
			yMax = 1200;
			fontSize = TEXT_48;
			numFont = NUM_28;
			break;
	}
}

void getCursor(void)
{
	int cursor_size = xMax/80;
	custom_cursor = create_bitmap( cursor_size, cursor_size);
	clear(custom_cursor);

	int w = custom_cursor->w;
	int h = custom_cursor->h;

	int y = h/10;
	for (int i = 0; i <= y; i++)
	{
		line(custom_cursor, 0, 0+i, w, h-i, 40);
		line(custom_cursor, 0+i , 0, w-i, h, 40);
		line(custom_cursor, 0+i , h, w-i, 0, 40);
		line(custom_cursor, 0 , h-i, w, 0+i, 40);
	}

	set_mouse_sprite(custom_cursor);
	set_mouse_sprite_focus(w/2, h/2);
}

char *listbox_getter(int index, int *list_size)
{
   static char *strings[] =
   {
      "640 X 480",  "800 X 600",   "1024 X 768",   "1280 X 1024", "1600 X 1200"
   };

   if (index < 0) {
      *list_size = 5;
      return NULL;
   }
   else
      return strings[index];
}